// VinciaRootGraph.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Function definitions (not found in the header) for the VinciaRootGraph class 

// Include the VinciaRootGraph header
#include "VRgraph.h"

namespace Vincia {
  
  //=====================================================================
  // VRgraph Member function definitions 

  //-----------------------------------------------------------------
  // update():
  // Update graph

  bool VRgraph::update(bool normalizeIn, double normFacIn, 
                       double sigmaStat, double scatterX) { 
    
    // Check histogram not empty
    if (hisPtrSav[0]->GetEntries() <= 0) return false;
    
    // Update
    sumWeights   = 0.0;
    // Include overflow bins in normalization
    int nBin = hisPtrSav[0]->GetNbinsX();
    int firstBin = 0;
    int nPoints  =  -1;
    for (int iBin = 0; iBin <= nBin; ++iBin) {
      double x    = hisPtrSav[0]->GetBinCenter(iBin);
      double exLo = x - hisPtrSav[0]->GetBinLowEdge(iBin);
      double exHi = hisPtrSav[0]->GetBinWidth(iBin) - exLo;
      // Apply scatter in x (so as not to plot in exactly same point)
      if (scatterX > 0.) x += scatterX * exHi;
      else x += scatterX * exLo;
      double y    = hisPtrSav[0]->GetBinContent(iBin);
      double ey   = sigmaStat * hisPtrSav[0]->GetBinError(iBin); 
      if (iBin == firstBin && y == 0.0) {
        ++firstBin;
        continue;
      }
      else if (y == 0. && ey == 0.) continue;
      // Count up graph point number from first non-overflow bin
      else if (iBin >= 1) {
        ++nPoints;
      }
      // Add uncertainty variations in quadrature with statistical
      double eyOutHi = 0.;
      double eyOutLo = 0.;
      if (hasOuter()) {
        for (int iHis = 1; iHis < int(hisPtrSav.size()); ++iHis) {
          // Check if same bins, else give error and switch off uncertainties
          if (abs(hisPtrSav[iHis]->GetBinCenter(iBin) - x) > TINY || 
              abs(x - hisPtrSav[iHis]->GetBinLowEdge(iBin) - exLo) > TINY ||
              abs(hisPtrSav[iHis]->GetBinWidth(iBin) - (exLo+exHi)) > TINY) {
            cout<<"(VRgraph): incompatible bins for uncertainty histogram "
                <<iHis<<" for "<<hisPtrSav[0]->GetName()<<endl;
            hasOuterSav = false;
            break;
          }
          // Compute error bracket
          double yBand = hisPtrSav[iHis]->GetBinContent(iBin);
          if (yBand > y) eyOutHi = max(eyOutHi,yBand-y);
          else eyOutLo = max(eyOutLo,y-yBand);            
        }	  
        eyOutHi = sqrt(pow(ey,2)+pow(eyOutHi,2));
        eyOutLo = sqrt(pow(ey,2)+pow(eyOutLo,2));
        if (iBin >= 1) {                        
          outer.SetPoint(nPoints,x,y);          
          outer.SetPointError(nPoints,exLo,exHi,eyOutLo,eyOutHi);          
        }
      }
      // Compute sum of weights
      double dx = exLo + exHi;
      sumWeights += y * dx;
      // Add content of overflow bin to sum of weights
      if (iBin == nBin) sumWeights += hisPtrSav[0]->GetBinContent(iBin+1) * dx;
      // Don't include underflow bin in graph        
      if (iBin == 0) continue;
      SetPoint(nPoints,x,y);
      SetPointError(nPoints,exLo,exHi,ey,ey);
    }
    
    // Check if normalizing to something
    if (normalizeIn && sumWeights != 0.0 && normFacIn != 0.0) {
      for (int iPoint = 0; iPoint <= nPoints; ++iPoint) {
        double yRaw, eyRawHi, eyRawLo, x;
        GetPoint(iPoint,x,yRaw);
        eyRawHi = GetErrorYhigh(iPoint);
        eyRawLo = GetErrorYlow(iPoint);
        SetPoint(iPoint,x,normFacIn*yRaw/sumWeights);
        SetPointEYhigh(iPoint,normFacIn*eyRawHi/sumWeights);
        SetPointEYlow(iPoint,normFacIn*eyRawLo/sumWeights);
        outer.GetPoint(iPoint,x,yRaw);
        eyRawHi = outer.GetErrorYhigh(iPoint);
        eyRawLo = outer.GetErrorYlow(iPoint);
        outer.SetPoint(iPoint,x,normFacIn*yRaw/sumWeights);
        outer.SetPointEYhigh(iPoint,normFacIn*eyRawHi/sumWeights);
        outer.SetPointEYlow(iPoint,normFacIn*eyRawLo/sumWeights);
      }	
    }
    return true;
  }      
  
  // Copy color scheme from another VRgraph
  void VRgraph::setColors(VRgraph* graph) {      
    SetMarkerSize(graph->GetMarkerSize());
    SetMarkerStyle(graph->GetMarkerStyle());
    SetMarkerColor(graph->GetMarkerColor());
    SetLineStyle(graph->GetLineStyle());
    SetLineColor(graph->GetLineColor());
    SetLineWidth(graph->GetLineWidth());
    SetFillColor(graph->GetFillColor());
    SetFillStyle(graph->GetFillStyle());
    outer.SetMarkerSize(graph->getOuter()->GetMarkerSize());
    outer.SetMarkerStyle(graph->getOuter()->GetMarkerStyle());
    outer.SetMarkerColor(graph->getOuter()->GetMarkerColor());
    outer.SetLineStyle(graph->getOuter()->GetLineStyle());
    outer.SetLineColor(graph->getOuter()->GetLineColor());
    outer.SetLineWidth(graph->getOuter()->GetLineWidth());
    outer.SetFillColor(graph->getOuter()->GetFillColor());
    outer.SetFillStyle(graph->getOuter()->GetFillStyle());
  }

  //-----------------------------------------------------------------
  // setColors():
  // Set colors according to predefined color schemes, defined here

  void VRgraph::setColors(int iScheme) {
    // Define color schemes 
    Int_t lineCol, statCol, totCol;
    Int_t lineStyle, markCol, markStyle;
    double markSize(0.4), lineWidth(1.);
    // Data : black-and-green scheme
    if (iScheme == 0) {
      markCol   =  1;
      markStyle = 21;
      lineStyle =  1;
      lineCol   =  1;
      statCol   = kGreen - 9;
      totCol    = kGreen - 4;
      markSize  = 0.6;
      outer.SetLineColor(lineCol);
      outer.SetLineWidth(lineWidth);
      outer.SetLineStyle(1);
    } 
    // Data : black-and-yellow scheme
    else if (iScheme == -1) {
      markCol   =  1;
      markStyle = 21;
      lineStyle =  1;
      lineCol   =  1;
      statCol   = 90;
      totCol    = 91;
      markSize  = 0.6;
      outer.SetLineColor(lineCol);
      outer.SetLineWidth(lineWidth);
      outer.SetLineStyle(1);
    } 
    // Data : two-sigma grey-and-yellow scheme
    else if (iScheme == -2) {
      markCol   = 16;
      markStyle = 21;
      lineStyle =  0;
      lineCol   = 16;
      statCol   = kYellow-9;
      totCol    = kYellow;
      markSize  = 0.6;
      outer.SetLineColor(lineCol);
      outer.SetLineWidth(lineWidth);
      outer.SetLineStyle(0);
    } 
    else if (iScheme == 1) {
      lineStyle =  1; 
      lineCol   = 61;
      statCol   = 64;
      totCol    = 66;
      markStyle = 20;
      markSize  = 0.4;
      markCol   = lineCol;
      lineWidth = 1.8;
    } 
    else if (iScheme == 2) {
      lineStyle =  1; 
      lineCol   = 2;
      statCol   = 50;
      totCol    = 46;
      markCol   = lineCol;
      markStyle = 25;
      markSize  = 0.58;
      lineWidth = 1.0;
    }
    else if (iScheme == 3) {
      lineStyle =  2; 
      lineCol   =  kMagenta+1;
      statCol   =  kMagenta-7;
      totCol    =  kMagenta-5;
      markCol   = lineCol;
      markStyle = 26;
      lineWidth = 1.5;
      markSize  = 0.58;
    }
    else if (iScheme == 4) {
      lineStyle =  3;
      lineCol   = kYellow + 3;
      statCol   = kYellow - 3;
      totCol    = kOrange + 2;
      markCol   = lineCol;
      markStyle = 33;
      lineWidth = 1.5;
      markSize  = 0.48;
    }
    else {
      lineStyle =  4;
      lineCol   = 32;
      statCol   = 30;
      totCol    = 29;
      markCol   = lineCol;
      markStyle =  32;
      lineWidth = 1.0;
      markSize  = 0.58;
    }
    SetMarkerSize(markSize);
    SetMarkerStyle(markStyle);
    SetMarkerColor(markCol);
    SetLineStyle(lineStyle);
    SetLineColor(lineCol);
    SetLineWidth(lineWidth);
    SetFillColor(statCol);
    outer.SetFillColor(totCol);
  }

  //=====================================================================
  // VRgraphData Member function definitions 

  //-----------------------------------------------------------------
  // update():
  // Update graph
  // Normalize to normFacIn
  // Multiply stat *and* sys errors by sigmaTot (eg for 2-sigma bands)

  bool VRgraphData::update(bool normalizeIn, double normFacIn, 
                           double sigmaTot, double) {
    if (dataFilePtr == 0) {
      cout<<"Error: cannot update dataGraph without dataFilePtr"<<endl;
      return false;
    }
    sumWeights   = dataFilePtr->getSumWeights();
    if (sumWeights == 0.0) sumWeights = 1.0;
    int nBin    = dataFilePtr->size();
    for (int iBin = 0; iBin < nBin; ++iBin) {
      double x    = dataFilePtr->getXC(iBin);
      double exLo = x - dataFilePtr->getXL(iBin);
      double exHi = dataFilePtr->getXR(iBin) - x;
      double y    = dataFilePtr->getY(iBin);
      double eyTotHi = dataFilePtr->getUtotHi(iBin);
      double eyTotLo = dataFilePtr->getUtotLo(iBin);
      double eyStatHi = dataFilePtr->getUstatHi(iBin);
      double eyStatLo = dataFilePtr->getUstatLo(iBin);        
      double eySysHi  = sqrt(max(0.0,pow(eyTotHi,2)-pow(eyStatHi,2)));
      double eySysLo  = sqrt(max(0.0,pow(eyTotLo,2)-pow(eyStatLo,2)));
      // Scale errors by sigmaTot (to show nSigma contours)
      eyStatHi *= sigmaTot;
      eyStatLo *= sigmaTot;
      eySysHi  *= sigmaTot;
      eySysLo  *= sigmaTot;
      eyTotHi = sqrt(pow(eyStatHi,2) + pow(eySysHi,2));
      eyTotLo = sqrt(pow(eyStatLo,2) + pow(eySysLo,2));
      SetPoint(iBin,x,y);
      SetPointError(iBin,exLo,exHi,eyStatLo,eyStatHi);
      outer.SetPoint(iBin,x,y);          
      outer.SetPointError(iBin,exLo,exHi,eyTotLo,eyTotHi);          
    }
    
    // Check if normalizing to something
    if (normalizeIn && sumWeights != 0.0 && normFacIn != 0.0) {
      for (int iBin = 0; iBin < nBin; ++iBin) {
        double yRaw, eyRawHi, eyRawLo, x;
        GetPoint(iBin,x,yRaw);
        eyRawHi = GetErrorYhigh(iBin);
        eyRawLo = GetErrorYlow(iBin);
        SetPoint(iBin,x,normFacIn*yRaw/sumWeights);
        SetPointEYhigh(iBin,normFacIn*eyRawHi/sumWeights);
        SetPointEYlow(iBin,normFacIn*eyRawLo/sumWeights);
        eyRawHi = outer.GetErrorYhigh(iBin);
        eyRawLo = outer.GetErrorYlow(iBin);
        outer.SetPoint(iBin,x,normFacIn*yRaw/sumWeights);
        outer.SetPointEYhigh(iBin,normFacIn*eyRawHi/sumWeights);
        outer.SetPointEYlow(iBin,normFacIn*eyRawLo/sumWeights);
      }
    }
    return true;
  }
  
  //=====================================================================
  // VRgraphHis Member function definitions 

  //-----------------------------------------------------------------
  // update():
  // Update graph

  bool VRgraphHis::update(bool normalizeIn, double normFacIn, 
                           double sigmaStat, double) {
    if (hisFilePtr == 0) {
      cout<<"Error: cannot update dataGraph without hisFilePtr"<<endl;
      return false;
    }
    sumWeights   = hisFilePtr->getSumWeights();
    if (sumWeights == 0.0) sumWeights = 1.0;
    int nBin    = hisFilePtr->size();
    for (int iBin = 0; iBin < nBin; ++iBin) {
      double x    = hisFilePtr->getXC(iBin);
      double exLo = x - hisFilePtr->getXL(iBin);
      double exHi = hisFilePtr->getXR(iBin) - x;
      double y    = hisFilePtr->getY(iBin);
      double eyTotHi = hisFilePtr->getUtotHi(iBin);
      double eyTotLo = hisFilePtr->getUtotLo(iBin);
      double eyStatHi = hisFilePtr->getUstatHi(iBin);
      double eyStatLo = hisFilePtr->getUstatLo(iBin);        
      double eySysHi  = sqrt(max(0.0,pow(eyTotHi,2)-pow(eyStatHi,2)));
      double eySysLo  = sqrt(max(0.0,pow(eyTotLo,2)-pow(eyStatLo,2)));
      // Scale stat error by sigmaStat (to show nSigma contours)
      eyStatHi *= sigmaStat;
      eyStatLo *= sigmaStat;
      eyTotHi = sqrt(pow(eyStatHi,2) + pow(eySysHi,2));
      eyTotLo = sqrt(pow(eyStatLo,2) + pow(eySysLo,2));
      SetPoint(iBin,x,y);
      SetPointError(iBin,exLo,exHi,eyStatLo,eyStatHi);
      outer.SetPoint(iBin,x,y);          
      outer.SetPointError(iBin,exLo,exHi,eyTotLo,eyTotHi);          
    }
    
    // Check if normalizing to something
    if (normalizeIn && sumWeights != 0.0 && normFacIn != 0.0) {
      for (int iBin = 0; iBin < nBin; ++iBin) {
        double yRaw, eyRawHi, eyRawLo, x;
        GetPoint(iBin,x,yRaw);
        eyRawHi = GetErrorYhigh(iBin);
        eyRawLo = GetErrorYlow(iBin);
        SetPoint(iBin,x,normFacIn*yRaw/sumWeights);
        SetPointEYhigh(iBin,normFacIn*eyRawHi/sumWeights);
        SetPointEYlow(iBin,normFacIn*eyRawLo/sumWeights);
        eyRawHi = outer.GetErrorYhigh(iBin);
        eyRawLo = outer.GetErrorYlow(iBin);
        outer.SetPoint(iBin,x,normFacIn*yRaw/sumWeights);
        outer.SetPointEYhigh(iBin,normFacIn*eyRawHi/sumWeights);
        outer.SetPointEYlow(iBin,normFacIn*eyRawLo/sumWeights);
      }
    }
    return true;
  }
  
  //=====================================================================
  // VRgraphNorm Member function definitions 

  //-----------------------------------------------------------------
  // update(): 
  // Update graph

  bool VRgraphNorm::update(bool, double, double sigmaIn, double) {
    
    // Sanity checks
    if (!hasNumerator || !hasDenominator) return false;
    if (numeratorPtr->GetN() <= 0) return false; 
    if (denominatorPtr->GetN() <= 0) return false; 
    
    // 
    hasOuterSav = numeratorPtr->hasOuter();
    
    // Zero sums for chi2 calculations
    double sumN5sigma2 = 0.0;
    double sumMCsigma2 = 0.0;
    int    nPointsSigma2 = 0;

    // Compute normalized histogram (using binning from denominator)
    int nBin    = denominatorPtr->GetN();
    int nPoints = -1;
    int iFirst  = 0;
    int iLast   = nBin -1;
    for (int iBin = iFirst; iBin <= iLast; ++iBin) {
      
      // Extract bin to use for normalization
      double xC, yDenom, exLo, exHi, xL, xR;
      denominatorPtr->GetPoint(iBin,xC,yDenom);        
      // Sanity check
      if (yDenom == 0.0) continue;
      double eyDenomNhi = denominatorPtr->getOuter()->GetErrorYhigh(iBin)/yDenom;
      double eyDenomNlo = denominatorPtr->getOuter()->GetErrorYlow(iBin)/yDenom;
      exLo = denominatorPtr->GetErrorXlow(iBin);
      exHi = denominatorPtr->GetErrorXhigh(iBin);
      xL = xC - exLo;
      xR = xC + exHi;
      // Loop over numerator bins and rebin like denominator
      double sumw     = 0.0;
      double sumy     = 0.0;
      // Statistical MC generation errors
      double sumErr2  = 0.0;
      // Note: up/down variation curves must be added linearly!
      double sumUpper = 0.0;
      double sumLower = 0.0;
      VRgraph* h=numeratorPtr;
      TGraphAsymmErrors* hU=numeratorPtr->getOuter();
      int nh = h->GetN();
      // Check plot range
      double xMin, xMax, yDum;
      h->GetPoint(0,xMin,yDum);
      xMin -= h->GetErrorXlow(0);
      h->GetPoint(nh-1,xMax,yDum);
      xMax +=  h->GetErrorXlow(nh-1);
      if (xL >= xMax) continue;
      if (xR <= xMin) continue;
      // Loop over bins
      double xRprev = xL;
      double bwPrev = 1.;
      for (int hBin=0; hBin<nh; ++hBin){
        // Get central values for this numerator bin
        double xNum, yNum, xNumU, yNumU;
        h->GetPoint(hBin,xNum,yNum);
        hU->GetPoint(hBin,xNumU,yNumU);       
        double xLnum = xNum - h->GetErrorXlow(hBin);
        double xRnum = xNum + h->GetErrorXhigh(hBin);
        double bw = xRnum-xLnum;
        // Check if not reached denominator bin left edge
        if (xRnum <= xL) continue;
        // Check if already passed denominator bin right edge
        if (xLnum >= xR) break;    
        // Sanity check
        if (bw <= 0.0) continue;
        double w = 1.0;
        // Subtract partial bin overlap
        w -= max(0.,xL-xLnum)/bw;
        w -= max(0.,xRnum-xR)/bw;
        w = max(w,0.0);
        sumw += w;
        // Add skipped empty bins (on the left)
        sumw += max(0.0,(xLnum-xRprev))/bw;
        xRprev = xRnum;
        bwPrev = bw;
        // Uncertainties
        double uStat   = 0.5*(h->GetErrorYlow(hBin)+h->GetErrorYhigh(hBin));
        double ySysHi  = yNumU + sqrt(max(0.0,pow(hU->GetErrorYhigh(hBin),2) 
                                          - pow(uStat,2)));
        double ySysLo  = yNumU - sqrt(max(0.0,pow(hU->GetErrorYlow(hBin),2) 
                                          - pow(uStat,2)));
        sumy     += w*yNum;            
        sumErr2  += w*pow(uStat,2);
        sumUpper += w*ySysHi;
        sumLower += w*ySysLo;
      }  	  
      // Sanity check
      if (sumw <= 0.) continue;
      // Add skipped empty bins between last and right edge
      sumw += max(0.0,xR-xRprev)/bwPrev;
      
      // Compute average values and total stat error
      double yTh   = sumy/sumw;          
      double eyThStat  = sigmaIn * sqrt(sumErr2)/sumw;
      double yVarUpper = sumUpper/sumw;
      double yVarLower = sumLower/sumw;
      double yVarTh    = 0.5*(yVarUpper+yVarLower);
      double eyVarTh   = yVarUpper-yVarTh;
      
      // Normalize by data
      Double_t yThN     = yTh/yDenom;
      Double_t eyThN    = eyThStat/yDenom;          
      Double_t yVarThN  = yVarTh/yDenom;
      // Add stat error in quadrature for normalized total uncertainty
      Double_t eyVarThN = sqrt(pow(eyVarTh,2) + pow(eyThStat,2))/yDenom;        
      
      // Compute chi2 with 5% blanket uncertainty
      if (yThN != 0.0 && eyThN > 0.) {
        ++nPointsSigma2;
        double n5sigma2 = ( yThN > 1.0 ) 
          ? pow(yThN - 1.0,2.)/(pow(eyThN,2.)+pow(eyDenomNhi,2.)+pow(0.05,2.))
          : pow(yThN - 1.0,2.)/(pow(eyThN,2.)+pow(eyDenomNlo,2.)+pow(0.05,2.));
        double sigma2mc = pow(eyThN,2.)
          /(pow(eyThN,2.)+pow(min(eyDenomNhi,eyDenomNlo),2.)+pow(0.05,2.));
        sumN5sigma2 += n5sigma2;
        sumMCsigma2 += sigma2mc;
      }
      
      // Limit to max +/- 50% (to clip plot)
      double eyThNhi = (yThN + eyThN > 1.5) ? max(0.0,1.5-yThN) : eyThN;
      double eyThNlo = (yThN - eyThN < 0.5) ? max(0.0,yThN-0.5) : eyThN;
      double eyVarThNhi = (yVarThN + eyVarThN > 1.5) ? max(0.0,1.5-yVarThN)
        : eyVarThN;
      double eyVarThNlo = (yVarThN - eyVarThN < 0.5) ? max(0.0,yVarThN-0.5)
        : eyVarThN;      
      
      // Set point value and errors;
      ++nPoints;
      SetPoint(nPoints,xC,yThN);
      SetPointError(nPoints,exLo,exHi,eyThNlo,eyThNhi);
      outer.SetPoint(nPoints,xC,yVarThN);          
      outer.SetPointError(nPoints,exLo,exHi,eyVarThNlo,eyVarThNhi);          

    }

    // Save average chi2 value and average relative uncertainy from MC stats
    if (nPoints >= 1) {
      chi2sav       = sumN5sigma2/nPointsSigma2;
      chi2errSav    = sumMCsigma2/nPointsSigma2; 
    } else {
      chi2sav       = 0.0;
      chi2errSav    = 1.0;
    }

    // Return
    return true;
  }
    
  //=====================================================================
  // VRgraphVar Member function definitions 

  //-----------------------------------------------------------------
  // update():
  // Update graph

  bool VRgraphVar::update(bool, double, double, double) {
    
    // Sanity checks
    if (referencePtr->GetN() <= 0) return false; 
    
    // Compute difference histogram (using binning from reference)
    // and normalize by reference
    TH1D* hRef  = referencePtr->getTH1DPtr(0);
    TH1D* hVar  = referencePtr->getTH1DPtr(iVarSav);
    int nBin    = hRef->GetNbinsX();
    int nPoints = -1;
    int iFirst  = 1;
    int iLast   = nBin;
    // Loop over histogram bins
    for (int iBin = iFirst; iBin <= iLast; ++iBin) {
      double xRef = hRef->GetBinCenter(iBin);
      double yRef = hRef->GetBinContent(iBin);
      //double eRef = hRef->GetBinError(iBin);
      double yVar = hVar->GetBinContent(iBin);
      //double eVar = hVar->GetBinError(iBin);
      double yMin = min(yRef,yVar);
      double yMax = max(yRef,yVar);
      for (int iVar=1; iVar<int(referencePtr->nOuter()); ++iVar) {
        if (iVar == iVarSav) continue;
        double yTmp = referencePtr->getTH1DPtr(iVar)->GetBinContent(iBin);
        if (yTmp > yMax) yMax = yTmp;
        else if (yTmp < yMin) yMin = yTmp;
      }
      // Sanity check (require non-zero total uncertainty)
      if (yMax - yMin == 0.0) continue;
      
      // Is this an up or down variation?
      double yFrac = 0.;
      double eyFracHi = 0.;
      double eyFracLo = 0.;
      if (yVar >= yRef) {
        // Sanity check (require non-zero total up variation)
        if (yMax - yRef <= 0.0) continue;
        yFrac = (yVar - yRef)/(yMax - yRef);          
        //eyFracHi = sqrt(pow(eVar,2)+pow(eRef,2))/(yMax - yRef);
        //if (yFrac + eyFracHi > 1.0) eyFracHi = 1.0 - yFrac;
        eyFracLo = abs(yFrac);
      } else {
        if (yRef - yMin <= 0.0) continue;
        yFrac = (yVar - yRef)/(yRef - yMin);
        //eyFracLo = sqrt(pow(eVar,2)+pow(eRef,2))/(yRef - yMin);
        //if (yFrac - eyFracLo < -1.0) eyFracLo = -1.0 - yFrac;
        eyFracHi = abs(yFrac);
      }
      
      // Set point value and errors;
      ++nPoints;
      SetPoint(nPoints,xRef,yFrac);
      SetPointError(nPoints,0.,0.,eyFracLo,eyFracHi);
    }
    return true;
  }
  
  //-----------------------------------------------------------------
  // setStyle(): 
  // Set line style for each variation, according to predefined schemes
  // defined here

  void VRgraphVar::setStyle(int iVar) {
    if (iVar == 1) {
      SetLineStyle(1);
      SetLineWidth(1.0);        
      SetFillColor(kBlack);
    }
    else if (iVar == 2) {
      SetLineStyle(1);
      SetLineWidth(1.0);        
      SetLineColor(kBlue-10);
      SetFillColor(kBlue-10);
    }
    else if (iVar == 3) {
      SetLineStyle(1);
      SetLineWidth(1.0);        
      SetLineColor(kRed-10);
      SetFillColor(kRed-10);
    }
    else if (iVar == 4) {
      SetLineStyle(1);
      SetLineWidth(2.0);        
      SetLineColor(kBlue-9);
      SetFillColor(kBlue-9);
      SetFillStyle(3305);
    }
    else if (iVar == 5) {
      SetLineStyle(1);
      SetLineWidth(2.0);        
      SetLineColor(kRed-8);
      SetFillColor(kRed-8);
      SetFillStyle(3305);
    }
    else if (iVar == 6) {
      SetLineStyle(1);
      SetLineWidth(2.0);        
      SetLineColor(kBlue-7);
      SetFillColor(kBlue-7);
      SetFillStyle(0);
    }
    else if (iVar == 7) {
      SetLineStyle(1);
      SetLineWidth(2.0);        
      SetLineColor(kRed-6);
      SetFillColor(kRed-6);
      SetFillStyle(0);
    }
    else if (iVar == 8) {
      SetLineStyle(2);
      SetLineWidth(1.0);        
      SetLineColor(1);
      SetFillStyle(0);
    }
    else if (iVar == 9) {
      SetLineStyle(4);
      SetLineWidth(1.0);        
      SetLineColor(1);
      SetFillStyle(0);
    }
    else if (iVar == 10) {
      SetLineStyle(3);
      SetLineWidth(1.0);        
      SetLineColor(kBlue+2);
      SetFillColor(kBlue+2);
      SetFillStyle(0);
    }
    else if (iVar == 11) {
      SetLineStyle(3);
      SetLineWidth(2.0);        
      SetLineColor(kRed+2);
      SetFillColor(kRed+2);
      SetFillStyle(0);
    }
  }

} // End namespace Vincia 

